local cql = ClassicQuestLog

ClassicQuestLogSettings = {} -- global savedvar
ClassicQuestLogPerSettings = {} -- per-character savedvar

-- default settings for global savedvar
local globalDefaults = {
    LockWindow = false, -- Lock Window Position
    ShowResizeGrip = true, -- Show Resize Grip
    PerCharacterPosition = false, -- Per-Character Window Placement
    ShowLevels = false, -- Show Quest Levels
    ShowTooltips = false, -- Show Quest Tooltips
    ShowFromObjectiveTracker = true, -- Show Window With Objective Tracker
    ShowFromKeyBind = true, -- Show Window With Default Key Bind
    ShowFromMicroButton = true, -- Show Window With Micro Button
    ShowDefaultQuestPortrait = false, -- Show Default Portraits
    ShowMinimapButton = false, -- Show Minimap Button
    ShowExtraTypeDetails = false, -- Show Extra Quest Details
    HideCampaignProgress = false, -- Hide Campaign Progress
    HighContrast = false, -- High Contrast Mode

    MinimapPosition = 164, -- (not in options) the position of minimap button
    LastVersion = false, -- (not in options) the version of this addon (30000 for 3.0.0) last session

    XPos = false,
    YPos = false,
    Width = false,
    Height = false,

    -- UseCustomScale = false,
    -- CustomScale = 100,
}

-- default settings for per-character savedvar
local perDefaults = {
    CollapsedHeaders = {}, -- lookup table indexed by title of quest headers that are collapsed
    QuestHistory = {}, -- ordered list of questIDs in the order they were selected
}

-- returns savedvar,defaults table references for the given key
local function getvars(key)
    if perDefaults[key] then
        return ClassicQuestLogPerSettings, perDefaults
    else
        return ClassicQuestLogSettings, globalDefaults
    end
end

-- metatable must remain empty for this to reliably work; which it will because setter never rawsets
local function getter(self,key)
    local savedvar, defaults = getvars(key)
    if savedvar[key]==nil then -- if actual setting undefined, save a default value if one exists
        if defaults[key] and type(defaults[key])=="table" then
            savedvar[key] = CopyTable(defaults[key])
        else
            savedvar[key] = defaults[key]
        end
    end
    return savedvar[key]
end

-- setter never does a rawset so table remains empty and __index always fires
local function setter(self,key,value)
    local savedvar, defaults = getvars(key)
    if type(value)=="table" then
        savedvar[key] = CopyTable(value)
    else
        savedvar[key] = value
    end
end

cql.settings = setmetatable({},{__index = getter, __newindex = setter})
